/* interact.c
/*    An example of an interactive CAVE environment. The user is able to
/*   add spheres to the environment using the right wand button, to grab
/*   existing spheres using the left button, and to resize a grabbed sphere
/*   with the joystick. The data for the spheres are kept in a linked list
/*   in shared memory, and are updated by the main computation process.
*/

#include <cave.h>
#include <malloc.h>
#include <gl/sphere.h>

/* The data that will be shared between processes */
struct _balldata
	{
	float x,y,z;
	float radius;
	int grabbed;
	struct _balldata *next;
	};

void init_gl(void),draw_balls(struct _balldata **);
void check_add(struct _balldata **list,void *shared_arena);
struct _balldata * update_held(struct _balldata *ball);
struct _balldata * check_grab(struct _balldata **list);

main(int argc,char **argv)
{
 struct _balldata **ballList,*heldBall=NULL;
 void *shared_arena;
 CAVEConfigure(&argc,argv,NULL);
/* Create a 4 megabyte arena */
 shared_arena = CAVEUserSharedMemory(4*1024*1024);
/* Initialize the list pointer */
 ballList = (struct _balldata **) acalloc(1,sizeof(struct _balldata *),
						shared_arena);
 CAVEInit();
 CAVEInitApplication(init_gl,0);
 CAVEDisplay(draw_balls,1,ballList);
 while (!getbutton(ESCKEY))
	{
	check_add(ballList,shared_arena);
	if (heldBall)
		heldBall = update_held(heldBall);
	else
		heldBall = check_grab(ballList);
	sginap(1);
	}
 CAVEExit();
}


void check_add(struct _balldata **list,void *shared_arena)
{
 if (CAVEButtonChange(3) == -1)
	{
	struct _balldata *ball;
	float wandPos[3],wandFront[3];
	ball = (struct _balldata *) amalloc(sizeof(struct _balldata),
						shared_arena);
	if (!ball)
		{
		fprintf(stderr,"OUT OF MEMORY!\n");
		return;
		}
	CAVEGetPosition(CAVE_WAND,wandPos);
	CAVEGetVector(CAVE_WAND_FRONT,wandFront);
	ball->x = wandPos[0] + wandFront[0]*2;
	ball->y = wandPos[1] + wandFront[1]*2;
	ball->z = wandPos[2] + wandFront[2]*2;
	ball->radius = 1;
	ball->grabbed = 0;
	ball->next = *list;
	*list = ball;
	}
}


struct _balldata * update_held(struct _balldata *ball)
{
 if (!CAVEBUTTON1)
	{
	float wandPos[3];
	CAVEGetPosition(CAVE_WAND,wandPos);
	ball->x = wandPos[0];
	ball->y = wandPos[1];
	ball->z = wandPos[2];
	ball->grabbed = 0;
	return NULL;
	}
 if (CAVE_JOYSTICK_Y > .5)
	ball->radius *= 1.005;
 else if (CAVE_JOYSTICK_Y < -.5)
	ball->radius /= 1.005;
 return ball;
}


struct _balldata * check_grab(struct _balldata **list)
{
 if (CAVEButtonChange(1) == 1)
	{
	struct _balldata *ball;
	float wandPos[3];
	CAVEGetPosition(CAVE_WAND,wandPos);
	for (ball = *list; ball; ball = ball->next)
		{
		float dist;
#define SQR(x) ((x)*(x))
		dist = SQR(ball->x - wandPos[0]) + SQR(ball->y - wandPos[1]) + 
			SQR(ball->z - wandPos[2]);
		if (dist < SQR(ball->radius))
			{
			ball->grabbed = 1;
			return ball;
			}
		}
	}
 return NULL;
}


/* init_gl - initialize GL lighting & materials */
void init_gl(void)
{
 float redMaterial[] = { DIFFUSE, 1, 0, 0, LMNULL };
 lmdef(DEFLMODEL,1,0,NULL);
 lmbind(LMODEL,1);
 lmdef(DEFLIGHT,1,0,NULL);
 lmbind(LIGHT0,1);
 lmdef(DEFMATERIAL,1,0,redMaterial);
}


/* draw_balls - draw the balls, using the shared data for their positions
  & sizes */
void draw_balls(struct _balldata **ballList)
{
 struct _balldata *ball;
 float sphereParam[4];

 czclear(0,getgdesc(GD_ZMAX));

 lmbind(MATERIAL,1);

 for (ball = *ballList; ball; ball = ball->next)
	if (ball->grabbed)
		{
		sphereParam[0] = sphereParam[1] = sphereParam[2] = 0;
		sphereParam[3] = ball->radius;
		pushmatrix();
		 CAVEWandTransform();
		 sphdraw(sphereParam);
		popmatrix();
		}
	else
		{
		sphereParam[0] = ball->x;
		sphereParam[1] = ball->y;
		sphereParam[2] = ball->z;
		sphereParam[3] = ball->radius;
		sphdraw(sphereParam);
		}
}
