/* navigate1.c
/*    An example of navigating in a CAVE environment. The world is that
/*  of bounce.c. The user navigates using the wand joystick - JOYSTICK_Y
/*  controls the forward/back motion; JOYSTICK_X controls turning.
*/

#include <cave.h>
#include <malloc.h>
#include <gl/sphere.h>

/* The data that will be shared between processes */
struct _balldata
	{
	float y;
	};

void init_gl(void),draw_balls(struct _balldata *);
struct _balldata *init_shmem(void);
void compute(struct _balldata *);
void navigate(void);

main(int argc,char **argv)
{
 struct _balldata *ball;
 CAVEConfigure(&argc,argv,NULL);
 ball = init_shmem();
 CAVEInit();
 CAVEInitApplication(init_gl,0);
 CAVEDisplay(draw_balls,1,ball);
 while (!getbutton(ESCKEY))
	{
	navigate();
	compute(ball);
	sginap(1);
	}
 CAVEExit();
}


#define SPEED 5.0f  /* Max navigation speed in feet per second */

/* navigate - perform the navigation calculations. This checks the joystick
  state and uses that to move and rotate. The Y position of the joystick
  determines the speed of motion in the direction of the wand. The X position
  of the joystick determines the speed of rotation about the CAVE's Y axis.
  Joystick values in the range -.2 to .2 are ignored; this provides a dead
  zone to eliminate noise. The motion is scaled by dt, the time passed since
  the last call to navigate(), in order to maintain a smooth speed. */
void navigate(void)
{
 float jx=CAVE_JOYSTICK_X,jy=CAVE_JOYSTICK_Y,dt,t;
 static float prevtime = 0;
 t = CAVEGetTime();
 dt = t - prevtime;
 prevtime = t;
 if (fabs(jy)>0.2)
	{
	float wandFront[3];
	CAVEGetVector(CAVE_WAND_FRONT,wandFront);
	CAVENavTranslate(wandFront[0]*jy*SPEED*dt, wandFront[1]*jy*SPEED*dt,
			wandFront[2]*jy*SPEED*dt);
	}
 if (fabs(jx)>0.2)
	CAVENavRot(-jx*90.0f*dt,'y');
}


/* init_shmem - initializes shared memory. The data is allocated from a
  shared memory arena, and so will be common to all processes forked after
  this is called. */
struct _balldata *init_shmem(void)
{
 void *arena;
 struct _balldata *ball;
 arena = CAVEUserSharedMemory(1024);
 ball = acalloc(2,sizeof(struct _balldata),arena);
 return ball;
}


/* compute - compute new positions for the balls. The height of the balls
  is a function of the current CAVE time. */
void compute(struct _balldata *ball)
{
 float t = CAVEGetTime();
 ball[0].y = fabs(sin(t)) * 6 + 1;
 ball[1].y = fabs(sin(t*1.2)) * 4 + 1;
}


/* init_gl - initialize GL lighting & materials */
void init_gl(void)
{
 float redMaterial[] = { DIFFUSE, 1, 0, 0, LMNULL };
 float blueMaterial[] = { DIFFUSE, 0, 0, 1, LMNULL };
 lmdef(DEFLMODEL,1,0,NULL);
 lmbind(LMODEL,1);
 lmdef(DEFLIGHT,1,0,NULL);
 lmbind(LIGHT0,1);
 lmdef(DEFMATERIAL,1,0,redMaterial);
 lmdef(DEFMATERIAL,2,0,blueMaterial);
}


/* draw_balls - draw the two balls, using the shared data for their
  y coordinates */
void draw_balls(struct _balldata *ball)
{
 float sphereParam0[] = { 2, 4, -5, 1};
 float sphereParam1[] = { -2, 4, -5, 1};

 czclear(0,getgdesc(GD_ZMAX));

/* Apply the navigation transformation */
 CAVENavTransform();

 lmbind(MATERIAL,0);
 cpack(0xff088018);
 pushmatrix();
  rot(-90.0,'x');
  rectf(-100.0,-100.0,100.0,100.0);
 popmatrix();

 lmbind(MATERIAL,1);
 sphereParam0[1] = ball[0].y;
 sphdraw(sphereParam0);

 lmbind(MATERIAL,2);
 sphereParam1[1] = ball[1].y;
 sphdraw(sphereParam1);
}
